// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_WRITINGBUFFER_H
#define HU_VARADIISTVAN_SCPL_IO_WRITINGBUFFER_H
//------------------------------------------------------------------------------

#if 0 // The public interface of WritingBuffer

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

/**
 * A buffer used for writing into a buffered stream.  It cannot be
 * created directly, only acquired from the stream.
 *
 * The data to write should be stored in the buffer. Then write()
 * should be called. If it returns true, it indicates that all data
 * has been written. In that case the buffer is automatically emptied.
 */
class WritingBuffer : public Buffer, public FailableReference<WritingBuffer>
{
public:
    /**
     * Reset the buffer. It resets the offset too.
     */
    void reset();

    /**
     * Write the contents of the buffer using the associated buffered
     * stream.
     *
     * If the buffer is empty, the function returns indicating
     * success.
     *
     * If the complete remaining contents could be written, it also indicates
     * success, and the buffer is reset, ready to be used again.
     *
     * If not everything can be written and the write operation
     * indicates that it would block, this function returns failure
     * after the stream has been set up  to handle the corresponding
     * poll events.
     *
     * Otherwise, if the writing operation fails for some other
     * reason, the buffer's error code is set to the error's code and
     * false is returned.
     */
    bool write();
};

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------

#endif // 0, The public interface of WritingBuffer

//------------------------------------------------------------------------------

#include "hu/varadiistvan/scpl/config.h"

//------------------------------------------------------------------------------

#if TARGET_API_POSIX

#include "_posix/WritingBuffer.h"

//------------------------------------------------------------------------------

#elif TARGET_API_WIN32 || TARGET_API_WIN64

#include "_win32/WritingBuffer.h"

//------------------------------------------------------------------------------

#else

#error "Not implemented for this target!"

#endif

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_WRITINGBUFFER_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
